@@ -2,7 +2,7 @@
 ## -*- coding: utf-8 -*-
 
 import os,sys
-import time
+import time,datetime
 from os.path import expanduser
 from multiprocessing import Lock
 from multiprocessing.dummy import Pool as ThreadPool
@@ -59,7 +59,6 @@ class MultipartUpload(object):
         except ValueError as e:
             error(u"Invalid value,please check .wcscfg file")
             sys.exit()
-            
 
     def __need_retry(self,code):
         if code == 500 or code == -1:
@@ -84,12 +83,19 @@ class MultipartUpload(object):
         result_dict = dict(zip(['offset', 'code', 'ctx'], result))
         result_dict['size'] = size
         if result_dict['code'] == 200:
-            lock.acquire()
+            #lock.acquire()
             self.progress += size
             debug('Current block size: %d, total upload size: %d' % (int(size), self.progress))
-            lock.release()
+            #lock.release()
         self.recorder.set_upload_record(result_dict['offset'],result_dict)
 
+    def todo_record_upload_progress(self, w_result_list):
+        '''
+        :param w_result_list: 多线程使用的list数据（result，size）
+        :return:
+        '''
+        return self._record_upload_progress(w_result_list[0],w_result_list[1])
+
     def _records_parse(self, upload_id):
         records = self.recorder.get_upload_record()
         offsetlist = [i * (self.block_size) for i in range(0,self.blocknum)]
@@ -168,8 +174,8 @@ class MultipartUpload(object):
                 debug('make block fail,code :{0},message :{1}'.format(blkcode, blktext))
             else:
                 result = self._make_bput(f, blktext['ctx'], offset)
-        self._record_upload_progress(result,size)
-        return blkcode
+        #self._record_upload_progress(result,size)
+        return blkcode,result,size
     
     def _make_bput(self, f, ctx, offset):
         bputnum = 1
@@ -207,6 +213,18 @@ class MultipartUpload(object):
                 return 0
         return 1
 
+    def _is_complete_smart(self,result_list):
+        '''
+        :param result_list: 块上传的结果集合
+        :return:
+        '''
+        if len(result_list) < self.blocknum:
+            return 0
+        for result in result_list:
+            if result[0] != 200:
+                return 0
+        return 1
+
     def _get_failoffsets(self):
         offsetlist = [i *  int(self.block_size) for i in range(0, self.blocknum)]
         if self.results:
@@ -263,14 +281,15 @@ class MultipartUpload(object):
             debug('There are %d offsets need to upload' % (len(offsets)))
             debug('Now start upload file blocks') 
             if self.concurrency > 0:
-                pool = ThreadPool(self.concurrency)
-                pool.map(self._make_block, offsets)
-                pool.close()
-                pool.join()
-
+                with ThreadPool(self.concurrency) as pool:
+                    result_list  = pool.map(self._make_block, offsets)
+                for result_keys in result_list:
+                    blkcode,result,size = result_keys
+                    self._record_upload_progress(result,size)
             elif self.concurrency == 0:
                 for offset in offsets:
-                    return_code = self._make_block(offset)
+                    return_code ,result,size= self._make_block(offset)
+                    self._record_upload_progress(result,size)
                     if 400 <= int(return_code) <= 499:
                         debug('Single-Thread,attempt authentication failed,exit the task.')
                         sys.exit()
@@ -294,31 +313,29 @@ class MultipartUpload(object):
             self.recorder = UploadProgressRecorder(upload_id)
             offsets = self._records_parse(upload_id)
             debug('Uncomplete offsetlist: %s, uploadid: %s, complete size: %d' % (offsets, upload_id, self.progress))
-
         else:
             debug("Now start a new multipart upload task")
             self._initial_records()
             offsets = [i * (self.block_size) for i in range(0,self.blocknum)]
-
         if len(offsets) != 0:
             debug('There are %d offsets need to upload' % (len(offsets)))
             debug('Now start upload file blocks')
             if self.concurrency > 0:
-                pool = ThreadPool(self.concurrency)
-                pool.map(self._make_block, offsets)
-                pool.close()
-                pool.join()
-
+                with ThreadPool(self.concurrency) as pool:
+                    result_list = pool.map(self._make_block, offsets)
+                for result_keys in result_list:
+                    blkcode,result,size = result_keys
+                    self._record_upload_progress(result,size)
             elif self.concurrency == 0:
                 for offset in offsets:
-                    return_code = self._make_block(offset)
+                    #return_code = self._make_block(offset)
+                    return_code ,result,size= self._make_block(offset)
+                    self._record_upload_progress(result,size)
                     if 400 <= int(return_code) <= 499:
                         debug('Single-Thread,attempt authentication failed,exit the task.')
                         sys.exit()
             else:
                 raise ValueError('Invalid concurrency')
-                sys.exit()
-
         if self._is_complete():
             debug('Now all blocks have upload suc.')
             mkfile_result = self._make_file()
@@ -327,8 +344,7 @@ class MultipartUpload(object):
             else:
                 raise WcsSeriveError("Make file fail,please upload file again")
         else:
-            fail_list = self._get_failoffsets()
             upload_record = str(Config.tmp_record_folder) + self.uploadBatch
             debug('Sorry! Mulitpart upload fail,more detail see %s' % upload_record)
             # return self.results
-            raise WcsSeriveError("Multipart upload fail,please upload file again")
+            raise WcsSeriveError("Multipart upload fail,please upload file again")
\ No newline at end of file
